==> Formula_0.2-0.tar.gz: Formula/man/model.frame.Formula.Rd <==
\name{model.frame.Formula}

\alias{terms.Formula}
\alias{model.matrix.Formula}
\alias{model.frame.Formula}
\alias{model.part}
\alias{model.part.formula}
\alias{model.part.Formula}

\title{Model Frame/Matrix/Response Construction for Extended Formulas}

\description{
  Computation of model frames, model matrices, and model responses for
  extended formulas of class \code{Formula}.
}

\usage{
\method{model.frame}{Formula}(formula, data = NULL, \dots,
  lhs = NULL, rhs = NULL)
\method{model.matrix}{Formula}(object, data = environment(object), \dots,
  lhs = NULL, rhs = 1)
\method{terms}{Formula}(x, \dots,
  lhs = NULL, rhs = NULL)

model.part(object, \dots)
\method{model.part}{Formula}(object, data, lhs = 0, rhs = 0,
  drop = FALSE, \dots)
}

\arguments{
  \item{formula, object, x}{an object of class \code{Formula}.}
  \item{data}{a data.frame, list or environment containing the variables in
    \code{formula}. For \code{model.part} it needs to be the \code{model.frame}.}
  \item{lhs, rhs}{indexes specifying which elements of the left- and
    right-hand side, respectively, should be employed. \code{NULL}
    corresponds to all parts, \code{0} to none. At least one \code{lhs} or
    one \code{rhs} has to be specified.}
  \item{drop}{logical. Should the \code{data.frame} be dropped for single
    column data frames?}
  \item{\dots}{further arguments passed to the respective
    \code{formula} methods.}
}

\details{
  All three model computations leverage the corresponding standard methods.
  Additionally, they allow specification of the part(s) of the left- and right-hand side
  (LHS and RHS) that should be included in the computation.

  The idea underlying all three model computations is to extract a suitable
  \code{formula} from the more general \code{Formula} and then calling
  the standard \code{\link[stats]{model.frame}}, \code{\link[stats]{model.matrix}},
  and \code{\link[stats]{terms}} methods.
  
  More specifically, if the \code{Formula} has multiple parts on the RHS,
  they are collapsed, essentially replacing \code{|} by \code{+}. If there
  is only a single response on the LHS, then it is kept on the LHS.
  Otherwise all parts of the formula are collapsed on the RHS (because \code{formula}
  objects can not have multiple responses). Hence, for multi-response \code{Formula}
  objects, the (non-generic) \code{\link[stats:model.extract]{model.response}} does
  not give the correct results. To avoid confusion a new generic \code{model.part} 
  with suitable \code{formula} method is provided which can always
  be used instead of \code{model.response}. Note, however, that it has a different
  syntax: It requires the \code{Formula} object in addition to the readily
  processed \code{model.frame} supplied in data
  (and optionally the \code{lhs}). Also, it returns either a \code{data.frame} with
  multiple columns or a single column (dropping the \code{data.frame} property)
  depending on whether multiple responses are employed or not.
}

\seealso{\code{\link{Formula}},  \code{\link[stats]{model.frame}},
  \code{\link[stats]{model.matrix}}, \code{\link[stats]{terms}},
  \code{\link[stats:model.extract]{model.response}}}

\examples{
## artificial example data
set.seed(1090)
dat <- as.data.frame(matrix(round(runif(21), digits = 2), ncol = 7))
colnames(dat) <- c("y1", "y2", "y3", "x1", "x2", "x3", "x4")
for(i in c(2, 6:7)) dat[[i]] <- factor(dat[[i]] > 0.5, labels = c("a", "b"))
dat$y2[1] <- NA
dat

######################################
## single response and two-part RHS ##
######################################

## single response with two-part RHS
F1 <- Formula(log(y1) ~ x1 + x2 | I(x1^2))
length(F1)

## set up model frame
mf1 <- model.frame(F1, data = dat)
mf1

## extract single response
model.part(F1, data = mf1, lhs = 1, drop = TRUE)
model.response(mf1)
## model.response() works as usual

## extract model matrices
model.matrix(F1, data = mf1, rhs = 1)
model.matrix(F1, data = mf1, rhs = 2)

#########################################
## multiple responses and multiple RHS ##
#########################################

## set up Formula
F2 <- Formula(y1 + y2 | log(y3) ~ x1 + I(x2^2) | 0 + log(x1) | x3 / x4)
length(F2)

## set up full model frame
mf2 <- model.frame(F2, data = dat)
mf2

## extract responses
model.part(F2, data = mf2, lhs = 1)
model.part(F2, data = mf2, lhs = 2)
## model.response(mf2) does not give correct results!

## extract model matrices
model.matrix(F2, data = mf2, rhs = 1)
model.matrix(F2, data = mf2, rhs = 2)
model.matrix(F2, data = mf2, rhs = 3)

##########
## misc ##
##########

## Formulas with '.'
F3 <- Formula(y1 | y2 ~ .)
mf3 <- model.frame(F3, data = dat)
## without y1 or y2
model.matrix(F3, data = mf3)
## without y1 but with y2
model.matrix(F3, data = mf3, lhs = 1)
## without y2 but with y1
model.matrix(F3, data = mf3, lhs = 2)
}

\keyword{models}

==> Formula_1.0-0.tar.gz: Formula/man/model.frame.Formula.Rd <==
\name{model.frame.Formula}

\alias{terms.Formula}
\alias{model.matrix.Formula}
\alias{model.frame.Formula}
\alias{model.part}
\alias{model.part.formula}
\alias{model.part.Formula}

\title{Model Frame/Matrix/Response Construction for Extended Formulas}

\description{
  Computation of model frames, model matrices, and model responses for
  extended formulas of class \code{Formula}.
}

\usage{
\method{model.frame}{Formula}(formula, data = NULL, \dots,
  lhs = NULL, rhs = NULL)
\method{model.matrix}{Formula}(object, data = environment(object), \dots,
  lhs = NULL, rhs = 1)
\method{terms}{Formula}(x, \dots,
  lhs = NULL, rhs = NULL)

model.part(object, \dots)
\method{model.part}{Formula}(object, data, lhs = 0, rhs = 0,
  drop = FALSE, \dots)
}

\arguments{
  \item{formula, object, x}{an object of class \code{Formula}.}
  \item{data}{a data.frame, list or environment containing the variables in
    \code{formula}. For \code{model.part} it needs to be the \code{model.frame}.}
  \item{lhs, rhs}{indexes specifying which elements of the left- and
    right-hand side, respectively, should be employed. \code{NULL}
    corresponds to all parts, \code{0} to none. At least one \code{lhs} or
    one \code{rhs} has to be specified.}
  \item{drop}{logical. Should the \code{data.frame} be dropped for single
    column data frames?}
  \item{\dots}{further arguments passed to the respective
    \code{formula} methods.}
}

\details{
  All three model computations leverage the corresponding standard methods.
  Additionally, they allow specification of the part(s) of the left- and right-hand side
  (LHS and RHS) that should be included in the computation.

  The idea underlying all three model computations is to extract a suitable
  \code{formula} from the more general \code{Formula} and then calling
  the standard \code{\link[stats]{model.frame}}, \code{\link[stats]{model.matrix}},
  and \code{\link[stats]{terms}} methods.
  
  More specifically, if the \code{Formula} has multiple parts on the RHS,
  they are collapsed, essentially replacing \code{|} by \code{+}. If there
  is only a single response on the LHS, then it is kept on the LHS.
  Otherwise all parts of the formula are collapsed on the RHS (because \code{formula}
  objects can not have multiple responses). Hence, for multi-response \code{Formula}
  objects, the (non-generic) \code{\link[stats:model.extract]{model.response}} does
  not give the correct results. To avoid confusion a new generic \code{model.part} 
  with suitable \code{formula} method is provided which can always
  be used instead of \code{model.response}. Note, however, that it has a different
  syntax: It requires the \code{Formula} object in addition to the readily
  processed \code{model.frame} supplied in data
  (and optionally the \code{lhs}). Also, it returns either a \code{data.frame} with
  multiple columns or a single column (dropping the \code{data.frame} property)
  depending on whether multiple responses are employed or not.
}

\references{
Zeileis A, Croissant Y (2010). Extended Model Formulas in R: Multiple Parts and Multiple Responses.
  \emph{Journal of Statistical Software}, \bold{34}(1), 1--13.
  \url{http://www.jstatsoft.org/v34/i01/}.
}

\seealso{\code{\link{Formula}},  \code{\link[stats]{model.frame}},
  \code{\link[stats]{model.matrix}}, \code{\link[stats]{terms}},
  \code{\link[stats:model.extract]{model.response}}}

\examples{
## artificial example data
set.seed(1090)
dat <- as.data.frame(matrix(round(runif(21), digits = 2), ncol = 7))
colnames(dat) <- c("y1", "y2", "y3", "x1", "x2", "x3", "x4")
for(i in c(2, 6:7)) dat[[i]] <- factor(dat[[i]] > 0.5, labels = c("a", "b"))
dat$y2[1] <- NA
dat

######################################
## single response and two-part RHS ##
######################################

## single response with two-part RHS
F1 <- Formula(log(y1) ~ x1 + x2 | I(x1^2))
length(F1)

## set up model frame
mf1 <- model.frame(F1, data = dat)
mf1

## extract single response
model.part(F1, data = mf1, lhs = 1, drop = TRUE)
model.response(mf1)
## model.response() works as usual

## extract model matrices
model.matrix(F1, data = mf1, rhs = 1)
model.matrix(F1, data = mf1, rhs = 2)

#########################################
## multiple responses and multiple RHS ##
#########################################

## set up Formula
F2 <- Formula(y1 + y2 | log(y3) ~ x1 + I(x2^2) | 0 + log(x1) | x3 / x4)
length(F2)

## set up full model frame
mf2 <- model.frame(F2, data = dat)
mf2

## extract responses
model.part(F2, data = mf2, lhs = 1)
model.part(F2, data = mf2, lhs = 2)
## model.response(mf2) does not give correct results!

## extract model matrices
model.matrix(F2, data = mf2, rhs = 1)
model.matrix(F2, data = mf2, rhs = 2)
model.matrix(F2, data = mf2, rhs = 3)

##########
## misc ##
##########

## Formulas with '.'
F3 <- Formula(y1 | y2 ~ .)
mf3 <- model.frame(F3, data = dat)
## without y1 or y2
model.matrix(F3, data = mf3)
## without y1 but with y2
model.matrix(F3, data = mf3, lhs = 1)
## without y2 but with y1
model.matrix(F3, data = mf3, lhs = 2)
}

\keyword{models}
